;;==============================================================================;;
;;		Hodiny			;;
;;==============================================================================;;
;;									;;
;; Program:		Hodiny		;;
;; Code:		Jindra Fucik ( http://www.fucik.name/ )	;;
;; Platform:		Microchip PIC16F628A, 4 Mhz		;;
;; Date:		03.07.2013			;;
;; First release:	03.07.2013			;;
;; LastDate:		03.07.2013			;;
;;									;;
;;==============================================================================;;
;
; Program pro digitalni hodiny ktere umoznuji FastClock a zaroven maji vlastni RTC modul
;
; Processor PIC 16F628A running at 4MHz internal oscillator.
;
; Revisions:
;
; 03/07/2013	Start of writing code.

; ----- Definitions

#define	__VERNUM	0x01
#define	__VERREV	"A"
#define	__VERDAY	0x03
#define	__VERMONTH	0x07
#define	__VERYEAR	0x13

                list    p=16F628A,r=hex

	errorlevel -302
	errorlevel -306

	INCLUDE "P16F628A.INC"

	 __FUSES _BODEN_ON & _CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_OFF  & _INTRC_OSC_NOCLKOUT
	;__FUSES _BODEN_ON & _CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON  & _HS_OSC

#define	RAMINI0	0x020		; 80 bytes
#define	RAMINI1	0x0A0		; 80 bytes
#define	RAMINI2	0x120		; 48 bytes
#define	RAMINT	0x070		; 16 bytes
;
;**********************************************************************************************************************
; ----- Macros
;**********************************************************************************************************************
;
SEL_BANK_0:	macro	
	bcf	STATUS,RP0		; macros for data access
	bcf	STATUS,RP1
	endm
;
SEL_BANK_1:	macro	
	bsf	STATUS,RP0
	bcf	STATUS,RP1
	endm
;
SEL_BANK_2:	macro	
	bcf	STATUS,RP0
	bsf	STATUS,RP1
	endm
;
SEL_BANK_3:	macro	
	bsf	STATUS,RP0
	bsf	STATUS,RP1
	endm
;
#define	SEL_IBANK_0	bcf	STATUS,IRP

#define	SEL_IBANK_1	bsf	STATUS,IRP

#define	DNOP	goto	$+1

;            +---------------+
; ROW2   01 -| RA2   U   RA1 |- 18 ROW1
; SCL    02 -| RA3       RA0 |- 17 ROW0
; SDA    03 -| RA4       RA7 |- 16 BTN3 (XBus addr)
; BTN1   04 -| RA5       RA6 |- 15 BTN2
; GND    05 -| Vss       Vdd |- 14 +5V
; DCC    06 -| RB0/INT   RB7 |- 13 COL-Enable (74 238) 0=disable 1=enable
; Rx     07 -| RB1/Rx    RB6 |- 12 SCK 595 - Serial clock 0->1 clk
; Tx     08 -| RB2/Tx    RB5 |- 11 RCK 595 - Parallel write 0->1 clk
; RxTx   09 -| RB3       RB4 |- 10 SER Data 595
;            +---------------+

FXTAL	equ	D'4000000'	; Internal oscillator frequency
DEBOUNCE_TIME equ 0x02

RA_TRIS         equ	0xF0	; RA0-RA3 out; RA4,RA5 in; RA6,RA7 out
RB_TRIS         equ	0x03	; all port B is output
RA_INI          equ	0x81	; COL=0, SDA=1, SCL=1
RB_INI          equ	0x00	; RxTx=0(rec), SCK=0, RCK=0, 
INTC_INI	equ	0xC0	; PEIE - Prhepial int enable, global int enable
PIE1_INI	equ	0x02	; TMR2IE - Timer 2 int enable (every 1ms)
T2CON_INI equ	0x05	; Prescaler 1:4, Timer 2 enable
PR2_INI equ	0xF9	; 249 dec = 250 instruction = 1ms with prescaler
OPTION_INI	equ	0x02	; Option reg: timer prescaler 1:4, pull-ups
			; Port B Pull-up:       	0   enabled
			; External int edge:    	0   falling
			; Timer trigger source: 	0   clock
			; Timer trigger edge:   	0   rising
			; Prescaler assignment: 	0   Timer0
			; Prescaler value:	010   1:8
;
;**********************************************************************************************************************
; --- Pin assignement
;**********************************************************************************************************************
;
I2CPORT	equ	PORTA	; PortA. Key input
ROWPORT	equ	PORTA	; PortA. output for rows
; --- Port A
;
; RA0	ROW0
; RA1	ROW1
; RA2	ROW2
; RA3	SCL (clock out for real time clock)
; RA4	SDA (I/O for real time clock PCF8563)
; RA5	BTN1
; RA6	BTN2
; RA7	BTN3
;
SCL		equ	3
SDA		equ	4
#define BTN1i		PORTA,5
#define BTN2i		PORTA,6
#define BTN3i		PORTA,7

;
; --- Port B
;
; RB0	DCC
; RB1	Rx
; RB2	Tx
; RB3	RxTx (0=Rx 1=Tx)
; RB4	SER (data to 595)
; RB5	RCK 595 - Parallel write 0->1 clk
; RB6	SCK 595 - Serial clock 0->1 clk
; RB7	ROW-Enable (74 238) 0=disable 1=enable
;
#define SER		PORTB,4
#define RCK		PORTB,5
#define SCK		PORTB,6
#define ROWena	PORTB,7
;
; ----- Variables
;
;**********************************************************************************************************************
; --- Internal RAM Section
;**********************************************************************************************************************
;
; --- Top on all banks
;
INT_W	equ	RAMINT+0x00	; Interrupt variables.
INT_STAT	equ	RAMINT+0x01
INT_PCLATH	equ	RAMINT+0x02
INT_FSR	equ	RAMINT+0x03

I2CDATA	equ	RAMINT+0x04	; I2C variables
I2CADDR	equ	RAMINT+0x05


; --- Bank 0

SHDA		equ	RAMINI0+0x00	; Shadow for port A - to be able to use all port write
SHDI2C		equ	SHDA
FLAGS		equ	RAMINI0+0x01	; flags for DCC, XBus and RTC communication
CURSEC		equ	RAMINI0+0x02	; indicating current displayed second
CURMIN		equ	RAMINI0+0x03	; indicating current displayed minute
CURHR		equ	RAMINI0+0x04	; indicating current displayed hour
CURDOW		equ	RAMINI0+0x05	; indicating current displayed day of week

COUNTRD		equ	RAMINI0+0x06	; counter for next read of time from RTC (250x skip)

I2CCOUNT	equ	RAMINI0+0x07	; counter for bites
I2CTEMP		equ	RAMINI0+0x08	; temp for bits

FSPD		equ	RAMINI0+0x09	; Fast clock Speed
FMIN		equ	RAMINI0+0x0A	; Fast clock Minute
FHR 		equ	RAMINI0+0x0B	; Fast clock Hour
FDOW		equ	RAMINI0+0x0C	; Fast clock Day of week

BTN1		equ	RAMINI0+0x0D	; status of buttons - 1st row
BTN2		equ	RAMINI0+0x0E	; status of buttons - 2nd row
BTN3		equ	RAMINI0+0x0F	; status of buttons - 3rd row

FSTCOUNT	equ	RAMINI0+0x26	; Counter for fast clock
KEY_DEB		equ	RAMINI0+0x27	; key debounce time
FROMBCD		equ	RAMINI0+0x28	; For Bin 2 BCD conversion
TOBCD		equ	RAMINI0+0x29	; 
DPCOUNT		equ	RAMINI0+0x2A	; counter for double dot after hours
COUNT		equ	RAMINI0+0x2B
TEMP		equ	RAMINI0+0x2C	; 

INTTMP		equ	RAMINI0+0x2D		; temp inside interrupt
INTCNT		equ	RAMINI0+0x2E		; counter inside interrupt
CURROW		equ	RAMINI0+0x2F		; current displayed row

STATINI		equ	RAMINI0+0x30		; First output buffer location
STAT0		equ	RAMINI0+0x30		; Output buffer - row 1
STAT1		equ	RAMINI0+0x31		; Output buffer - row 2
STAT2		equ	RAMINI0+0x32		; Output buffer - row 3
STAT3		equ	RAMINI0+0x33		; Output buffer - row 4
STAT4		equ	RAMINI0+0x34		; Output buffer - row 5
STAT5		equ	RAMINI0+0x35		; Output buffer - row 6
STAT6		equ	RAMINI0+0x36		; Output buffer - row 7
STAT7		equ	RAMINI0+0x37		; Output buffer - row 8
STATEND		equ	RAMINI0+0x37		; Last output buffer location

;----- Flags

#define	NoClock	FLAGS,0	; No clock module detected
#define	RdTime	FLAGS,1	; Time to read time 0.25s
#define	I2CErr	FLAGS,2	; Error received from I2C bus

;----- Character set
CHAR_a	equ 0x0A
CHAR_P	equ 0x0B
CHAR_C	equ 0x0C
CHAR_o	equ 0x0D
CHAR_e	equ 0x0E
CHAR_U	equ 0x0F
CHAR_t	equ 0x10
CHAR_S	equ 0x11
CHAR_N	equ 0x12
CHAR_r	equ 0x13

SEG_0	equ	b'01111110'		; 0                  2
SEG_1	equ	b'00001100'		; 1                7   3
SEG_2	equ	b'10110110'		; 2                  8
SEG_3	equ	b'10011110'		; 3                6   4
SEG_4	equ	b'11001100'		; 4                  5   1
SEG_5	equ	b'11011010'		; 5
SEG_6	equ	b'11111010'		; 6
SEG_7	equ	b'00001110'		; 7
SEG_8	equ	b'11111110'		; 8
SEG_9	equ	b'11011110'		; 9
SEG_A	equ	b'11101110'		; A (a)
SEG_P	equ	b'11100110'		; B (P)
SEG_C	equ	b'01110010'		; C (C)
SEG_O	equ	b'10111000'		; D (o)
SEG_E	equ	b'11110010'		; E (e) 
SEG_U	equ	b'01111100'		; F (U)
SEG_T	equ	b'11110000'		; 10 (t)
SEG_S	equ	b'11011010'		; 11 (S)
SEG_N	equ	b'01101110'		; 12 (N)
SEG_R	equ	b'10100000'		; 13 (r) 
SEG_mi	equ	b'10000000'		; 13 (r) 
;
;**********************************************************************************************************************
; Program Section
;**********************************************************************************************************************
;
	org	0x000
PowerUp:
	clrf	INTCON	; Disable all interrupts
	clrf	PCLATH	; Tables on page 0
	clrf	STATUS	; reset flags
	goto	INIT
;
;**********************************************************************************************************************
; ISR (Interrupt Service Routines)
;**********************************************************************************************************************
;
	org	0x004
Interrupt:
	movwf	INT_W	; 1 Save registers W, STATUS, PCLATH, FSR
	swapf	STATUS,w	; 2
	clrf	STATUS	; 3 Ensure port / timer access in bank 0
	movwf	INT_STAT	; 4
	movf	PCLATH,w	; 
	movwf	INT_PCLATH	; save PCLATH, interrupt uses page 0
	clrf	PCLATH	; 
	movf	FSR,w
	movwf	INT_FSR

	bcf	PIR1,TMR2IF	; open int for timer2
	;bsf	NewRow		; indicate new row for main loop
	movf	CURROW,w		; prepare number of column to W
	sublw	STATEND			; add base of buffer
	movwf	FSR			; to FSR
	movf	INDF,w			; read data from indirect
;	upload data into 595
	movwf	INTTMP
	movlw	0x08
	movwf	INTCNT


IntNextBit:
		bcf SER
		btfsc	INTTMP,0
		bsf	SER
		bsf	SCK		; clock _/-
		rrf	INTTMP,f			; next bite
		bcf	SCK		; clock -\_
		decfsz	INTCNT,f
		goto	IntNextBit


	movf	SHDA,w			; read shadow register (for LED1 and I2C com)
	andlw	0xF8			; mask only upper part
	iorwf	CURROW,w		; add current row
	movwf	SHDA			; update it back into shadow (no necessarry)
	bcf	ROWena				; switch off leds, to avoid blinking
	movwf	ROWPORT			; new rows
	bsf	RCK					; upload data to the leds
	incf	CURROW,f		; next row
	bcf	RCK					; back
	movlw 0x07
	bsf	ROWena				; switch leds back on,
	andwf CURROW,f			; only 7 rows
	decf CURROW,w
	call BitPos				; prepare BitPos into W for button masks
	xorlw 0xFF				; negative - to remove
	andwf BTN1,f
	andwf BTN2,f
	andwf BTN3,f
	xorlw 0xFF				; back to positive to add bits
	btfsc BTN1i
	iorwf BTN1,f
	btfsc BTN2i
	iorwf BTN2,f
	btfsc BTN3i
	iorwf BTN3,f
	

	
IntEnd:
	decfsz	COUNTRD,f
	goto	IntNoRead
	movlw	d'250'
	movwf	COUNTRD
	bsf	RdTime
IntNoRead:

	movf	INT_FSR,w
	movwf	FSR
	movf	INT_PCLATH,w
	movwf	PCLATH
	swapf	INT_STAT,w
	movwf	STATUS
	swapf	INT_W,f
	swapf	INT_W,w
	retfie

;**********************************************************************************************************************
; Tables on first 256 bytes
;**********************************************************************************************************************
;
HexTo7Seg:
	andlw	0x0F
	addwf	PCL,f
	retlw	SEG_0		; 0                  1
	retlw	SEG_1		; 1                6   2
	retlw	SEG_2		; 2                  7
	retlw	SEG_3		; 3                5   3
	retlw	SEG_4		; 4                  4
	retlw	SEG_5		; 5
	retlw	SEG_6		; 6
	retlw	SEG_7		; 7
	retlw	SEG_8		; 8
	retlw	SEG_9		; 9
	retlw	SEG_A		; A (a)
	retlw	SEG_P		; B (P)
	retlw	SEG_C		; C (C)
	retlw	SEG_O		; D (o)
	retlw	SEG_E		; E (e) 
	retlw	SEG_U		; F (U)

BitPos:
	andlw	0x07
	addwf	PCL,f
	retlw	0x01
	retlw	0x02
	retlw	0x04
	retlw	0x08
	retlw	0x10
	retlw	0x20
	retlw	0x40
	retlw	0x80

DOWH:
	andlw	0x07
	addwf	PCL,f
	retlw	SEG_N		; Ne
	retlw	SEG_P		; Po
	retlw	SEG_U		; Ut
	retlw	SEG_S		; St
	retlw	SEG_C		; Ct
	retlw	SEG_P		; Pa
	retlw	SEG_S		; So
	retlw	SEG_mi		; --

DOWL:
	andlw	0x07
	addwf	PCL,f
	retlw	SEG_E		; Ne
	retlw	SEG_O		; Po
	retlw	SEG_T		; Ut
	retlw	SEG_T		; St
	retlw	SEG_T		; Ct
	retlw	SEG_A		; Pa
	retlw	SEG_O		; So
	retlw	SEG_mi		; --


SpeedSteps:
	andlw	0x1F
	addwf	PCL,f
	retlw	d'1'	; 1:0 = infinite steps per min
	retlw	d'240'	; 1:1 = 4*60=240 steps per min
	retlw	d'120'	; 1:2 = 240/2=120 steps per min
	retlw	d'80'	; 1:3 = 240/3=80 steps per min
	retlw	d'60'	; 1:4 = 240/4=60 steps per min
	retlw	d'48'	; 1:5 = 240/5=48 steps per min
	retlw	d'40'	; 1:6 = 240/6=40 steps per min
	retlw	d'34'	; 1:7 = 240/7=34 steps per min
	retlw	d'30'	; 1:8 = 240/8=30 steps per min
	retlw	d'27'	; 1:9 = 240/9=27 steps per min
	retlw	d'24'	; 1:10 = 240/10=24 steps per min
	retlw	d'22'	; 1:11 = 240/11=22 steps per min
	retlw	d'20'	; 1:12 = 240/12=20 steps per min
	retlw	d'18'	; 1:13 = 240/13=18 steps per min
	retlw	d'17'	; 1:14 = 240/14=17 steps per min
	retlw	d'16'	; 1:15 = 240/15=16 steps per min
	retlw	d'15'	; 1:16 = 240/16=15 steps per min
	retlw	d'14'	; 1:17 = 240/17=14 steps per min
	retlw	d'13'	; 1:18 = 240/18=13 steps per min
	retlw	d'13'	; 1:19 = 240/19=13 steps per min
	retlw	d'12'	; 1:20 = 240/20=12 steps per min
	retlw	d'11'	; 1:21 = 240/21=11 steps per min
	retlw	d'11'	; 1:22 = 240/22=11 steps per min
	retlw	d'10'	; 1:23 = 240/23=10 steps per min
	retlw	d'10'	; 1:24 = 240/24=10 steps per min
	retlw	d'10'	; 1:25 = 240/25=10 steps per min
	retlw	d'9'	; 1:26 = 240/26=9 steps per min
	retlw	d'9'	; 1:27 = 240/27=9 steps per min
	retlw	d'9'	; 1:28 = 240/28=9 steps per min
	retlw	d'8'	; 1:29 = 240/29=8 steps per min
	retlw	d'8'	; 1:30 = 240/30=8 steps per min
	retlw	d'7'	; 1:31 = 240/31=7 steps per min

;
	if ($ > d'255') 
		ERROR "  Tables exceded page 0.   If it works, why do you change it?   "
	endif
;
; ----- Initialization
;
INIT:
	clrf	RCSTA	; disable UART
	movlw   	RA_INI	; Set ports
	movwf   	PORTA                         
	movlw   	RB_INI
	movwf   	PORTB
	SEL_BANK_1
	movlw	RB_TRIS	; Set port B I/O configuration
	movwf   	TRISB
	movlw   	RA_TRIS         ; Set port A I/O configuration
	movwf   	TRISA
	movlw	PIE1_INI
	movwf	PIE1	; peripheral interrupts
	movlw   	OPTION_INI      ; PORTB pull-ups enabled, Timer configuration
	movwf   	OPTION_REG
	movlw	0x0B	; internal oscillator to 4MHz default
	movwf	PCON
	movlw	0x00	; VRR off voltage reference
	movwf	VRCON
	movlw	PR2_INI
	movwf	PR2
	SEL_BANK_0
	movlw	T2CON_INI
	movwf	T2CON
	movlw	0x07	; RA0, RA1, RA2 digital inputs
	movwf	CMCON                          
	movlw	RAMINI0	; Clear variables bank0
	movwf	FSR
ClearRAM:
	clrf	INDF
	incf	FSR,f
	btfss	FSR,7	
	goto	ClearRAM
;
	movlw	INTC_INI	; Set interrupts
	movwf	INTCON
         
	movlw	d'250'
	movwf	COUNTRD

	movlw	0xFF
	movwf	CURSEC		; FF in sec, min, hr indicate, that time need to be updated
	movwf	CURMIN
	movwf	CURHR
	bsf	RdTime

	bcf	PIR1,TMR2IF	; open int for timer2
	bsf	INTCON,GIE	; enable interupt
	bsf	T2CON,TMR2ON	; enable timer2

	bcf		NoClock
         
	movlw	0x0D
	movwf	I2CADDR		; CLKOUT control register = 0D
	clrf	I2CDATA		; disable CLKOUT
	bcf		I2CErr
	call	I2CWrite
	btfsc	I2CErr
	bsf		NoClock
         
;
;
;**********************************************************************************************************************
; --- Main Loop
;**********************************************************************************************************************
;

MainLoop:
	btfsc	RdTime
	call	ReadTime

	;btfsc	NewRow
	;call	DrawNextRow

	goto	MainLoop


;
;**********************************************************************************************************************
; --- Bin and BCD conversions
;**********************************************************************************************************************
;
BIN2BCD:
	bcf        STATUS,C        ; 16 bit bin to 4 digit BCD
	movlw        0x08
	movwf        COUNT
	clrf        FROMBCD
	;clrf        R1
LOOP8:
	rlf        TOBCD,f
	;rlf        MSB,f
	rlf        FROMBCD,f
	;rlf        R1,f
	decfsz        COUNT,f
	;goto        ADJDEC
	goto		ADJBCD
	return
;ADJDEC:
	;movlw        R0
	;call        ADJBCD
	;movlw        R1
	;call        ADJBCD
	;goto        LOOP8
ADJBCD:
	;movwf         FSR
	movlw         0x3
	;addwf         INDF,W
	addwf         FROMBCD,W
	movwf        TEMP
	btfsc        TEMP,3
	;movwf        INDF
	movwf        FROMBCD
	movlw        0x30
	;addwf        INDF,W
	addwf        FROMBCD,W
	movwf        TEMP
	btfsc        TEMP,7
	;movwf        INDF
	movwf        FROMBCD
	;return
	goto	LOOP8

BCDInc:
	movwf	FROMBCD
	movlw   0x67 
	addwf   FROMBCD,f
	movlw	0x60 
	btfss	STATUS,DC 
	movlw	0x66 
	btfss	STATUS,C 
	subwf	FROMBCD,f
	movf	FROMBCD,w
	return

BCDDec: 
	movwf	FROMBCD
	movlw   1 
	subwf   FROMBCD,f
	addwf   FROMBCD,w 
	movlw   0x06 
	btfsc	STATUS,C 
	movlw  0x66 
	btfsc	STATUS,DC 
	subwf	FROMBCD,f
	movf	FROMBCD,w
	return

BCDInc60:
	movwf	FROMBCD
	xorlw	0x59
	btfsc	STATUS,Z
	return
	movf	FROMBCD,w
	goto	BCDInc

BCDDec60:
	movwf	FROMBCD
	btfsc	STATUS,Z
	retlw	0x59
	goto	BCDDec

BCDInc24:
	movwf	FROMBCD
	xorlw	0x23
	btfsc	STATUS,Z
	return
	movf	FROMBCD,w
	goto	BCDInc

BCDDec24:
	movwf	FROMBCD
	btfsc	STATUS,Z
	retlw	0x23
	goto	BCDDec


ReadTime:
	;btfsc	NewRow	; New row have priority
	;return
	bcf		RdTime
	btfsc	NoClock
	goto	NoClockMsg
	btfsc	BTN2,0
	goto	PauseClock
	btfsc	BTN2,2
	goto	FastClock

RealClock:
	movf	KEY_DEB,w	; skip debounce time
	btfss	STATUS,Z
	goto	SkipBtnRD
	movf	BTN1,w
	btfsc	STATUS,Z
	goto	SkipBtn1RD

	btfss	BTN1,1
	goto	NoIncS
	movf	CURSEC,w
	andlw	0x7F
	call	BCDInc60
	movwf	CURSEC
	movwf	I2CDATA
	movlw	0x02		; sec
	movwf	I2CADDR		; Sec register = 02
	bcf		I2CErr
	call	I2CWrite
	bsf		CURSEC,7
NoIncS
	btfss	BTN1,0
	goto	NoDecS
	movf	CURSEC,w
	andlw	0x7F
	call	BCDDec60
	movwf	CURSEC
	movwf	I2CDATA
	movlw	0x02		; sec
	movwf	I2CADDR		; Sec register = 02
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoDecS:
	btfss	BTN1,3
	goto	NoIncM
	movf	CURMIN,w
	call	BCDInc60
	movwf	CURMIN
	movwf	I2CDATA
	movlw	0x03		; min
	movwf	I2CADDR		; Min register = 03
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoIncM
	btfss	BTN1,2
	goto	NoDecM
	movf	CURMIN,w
	call	BCDDec60
	movwf	CURMIN
	movwf	I2CDATA
	movlw	0x03		; min
	movwf	I2CADDR		; Min register = 03
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoDecM:
	btfss	BTN1,5
	goto	NoIncH
	movf	CURHR,w
	call	BCDInc24
	movwf	CURHR
	movwf	I2CDATA
	movlw	0x04		; hour
	movwf	I2CADDR		; Hour register = 04
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoIncH
	btfss	BTN1,4
	goto	NoDecH
	movf	CURHR,w
	call	BCDDec24
	movwf	CURHR
	movwf	I2CDATA
	movlw	0x04		; hour
	movwf	I2CADDR		; Hour register = 04
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoDecH:
	btfss	BTN1,7
	goto	NoIncD
	incf	CURDOW,f
	movlw	0x07
	xorwf	CURDOW,w
	andlw	0x07
	btfsc	STATUS,Z
	clrf	CURDOW
	movf	CURDOW,w
	movwf	I2CDATA
	movlw	0x06		; weekdays
	movwf	I2CADDR		; day of week register = 06
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoIncD
	btfss	BTN1,6
	goto	NoDecD
	movlw	0x07
	andwf	CURDOW,f
	decf	CURDOW,f
	movlw	0x06
	btfsc	CURDOW,3
	movwf	CURDOW
	movf	CURDOW,w
	movwf	I2CDATA
	movlw	0x06		; weekdays
	movwf	I2CADDR		; day of week register = 06
	bcf		I2CErr
	call	I2CWrite
	movlw	0xFF
	movwf	CURSEC
NoDecD:

	movlw	DEBOUNCE_TIME
	movwf	KEY_DEB
SkipBtn1RD:
	movf	BTN2,w
	andlw	b'00001010' ; buttons only on 2,1 and 2,3
	btfsc	STATUS,Z
	goto	ShowRealClock
	; read keys from BTN2
	btfss	BTN2,1
	goto	NoResetS
	clrf	CURSEC
	movlw	0x02		; sec + VL on bit 7 
	movwf	I2CADDR		; SEC register = 02
	movf	CURSEC,w
	movwf	I2CDATA
	bcf		I2CErr
	call	I2CWrite
NoResetS:
	btfss	BTN2,3
	goto	NoResetM
	clrf	CURMIN
	movlw	0x03		; minutes
	movwf	I2CADDR		; MIN register = 03
	movf	CURMIN,w
	movwf	I2CDATA
	bcf		I2CErr
	call	I2CWrite
NoResetM:	
	movlw	DEBOUNCE_TIME
	movwf	KEY_DEB
	goto	ShowRealClock
SkipBtnRD:
	decf	KEY_DEB,f
ShowRealClock:
	incf	DPCOUNT,f

	btfsc	DPCOUNT,1
	bsf		STAT3,0		; half sec for point
	btfss	DPCOUNT,1
	bcf		STAT3,0		; half sec for point

	btfsc	DPCOUNT,0
	bsf		STAT5,0		; half sec for point
	btfss	DPCOUNT,0
	bcf		STAT5,0		; half sec for point

	movlw	0x02
	movwf	I2CADDR		; SEC register = 02
	bcf		I2CErr
	call	I2CRead
	btfsc	I2CErr
	bsf		NoClock
	movf	I2CDATA,w
	xorwf	CURSEC,w
	btfsc	STATUS,Z
	return				; no change = no action
	movf	I2CDATA,w
	movwf	CURSEC

	swapf	CURSEC,w
	andlw	0x07		; only 3 bites valid for Sec 10
	call	HexTo7Seg
	movwf	STAT6
	movf	I2CDATA,w
	call	HexTo7Seg
	btfsc	CURSEC,7	; upper bit mean not synchro
	iorlw	0x01		; add DP if not synch
	movwf	STAT7

	movlw	0x03		; minutes
	movwf	I2CADDR		; MIN register = 03
	bcf		I2CErr
	call	I2CRead
	movf	I2CDATA,w
	movwf	CURMIN
	swapf	I2CDATA,w
	andlw	0x07		; only 3 bites valid for Min 10
	call	HexTo7Seg
	movwf	STAT4
	movf	I2CDATA,w
	call	HexTo7Seg
	movwf	STAT5
	btfsc	DPCOUNT,0
	bsf		STAT5,0		; half sec for point

	movlw	0x04		; hours
	movwf	I2CADDR		; HR register = 04
	bcf		I2CErr
	call	I2CRead
	movf	I2CDATA,w
	movwf	CURHR
	swapf	I2CDATA,w
	andlw	0x03		; only 2 bites valid for Hr 10
	call	HexTo7Seg
	movwf	STAT2
	movf	I2CDATA,w
	call	HexTo7Seg
	movwf	STAT3
	btfsc	DPCOUNT,1
	bsf		STAT3,0		; half sec for point

	movlw	0x06		; weekdays
	movwf	I2CADDR		; HR register = 04
	bcf		I2CErr
	call	I2CRead
	movf	I2CDATA,w
	movwf	CURDOW
	;movf	I2CDATA,w
	andlw	0x07		; only 3 bites valid for day of week
	call	DOWH
	movwf	STAT0
	movf	I2CDATA,w
	andlw	0x07		; only 3 bites valid for day of week
	call	DOWL
	movwf	STAT1

	return

PauseClock:
	incf	DPCOUNT,f

	btfsc	DPCOUNT,0
	bsf		STAT3,0		; half sec for point
	btfss	DPCOUNT,0
	bcf		STAT3,0		; half sec for point


	movlw	SEG_mi
	movwf	STAT6
	movwf	STAT7

	movf	FMIN,w	; mins
	movwf	TOBCD
	call	BIN2BCD
	swapf	FROMBCD,w
	andlw	0x07		; only 3 bites valid for Min 10
	call	HexTo7Seg
	movwf	STAT4
	movf	FROMBCD,w
	call	HexTo7Seg
	movwf	STAT5

	movf	FHR,w		; hours
	movwf	TOBCD
	call	BIN2BCD
	swapf	FROMBCD,w
	andlw	0x03		; only 2 bites valid for Hr 10
	call	HexTo7Seg
	movwf	STAT2
	movf	FROMBCD,w
	call	HexTo7Seg
	movwf	STAT3
	btfsc	DPCOUNT,0
	bsf		STAT3,0		; half sec for point

	movf	FDOW,w
	andlw	0x07		; only 3 bites valid for day of week
	call	DOWH
	movwf	STAT0
	movf	FDOW,w
	andlw	0x07		; only 3 bites valid for day of week
	call	DOWL
	movwf	STAT1
	return

FastClock:
	movf	KEY_DEB,w	; skip debounce time
	btfss	STATUS,Z
	goto	SkipFBtnRD
	movf	BTN1,w
	btfsc	STATUS,Z
	goto	SkipFBtn1RD

	btfss	BTN1,1
	goto	NoFIncS
	incf	FSPD,f
	movlw	0x1F
	andwf	FSPD,f
	movf	FSPD,w
	call	SpeedSteps
	movwf	FSTCOUNT
NoFIncS
	btfss	BTN1,0
	goto	NoFDecS
	decf	FSPD,f
	movlw	0x1F
	andwf	FSPD,f
	movf	FSPD,w
	call	SpeedSteps
	movwf	FSTCOUNT
NoFDecS:
	btfss	BTN1,3
	goto	NoFIncM
	incf	FMIN,f
	movlw	d'60'
	xorwf	FMIN,w
	btfsc	STATUS,Z
	clrf	FMIN
NoFIncM
	btfss	BTN1,2
	goto	NoFDecM
	decf	FMIN,f
	movlw	d'59'
	btfsc	FMIN,7
	movwf	FMIN	
NoFDecM:
	btfss	BTN1,5
	goto	NoFIncH
	incf	FHR,f
	movlw	d'24'
	xorwf	FHR,w
	btfsc	STATUS,Z
	clrf	FHR
NoFIncH
	btfss	BTN1,4
	goto	NoFDecH
	decf	FHR,f
	movlw	d'23'
	btfsc	FHR,7
	movwf	FHR	
NoFDecH:
	btfss	BTN1,7
	goto	NoFIncD
	incf	FDOW,f
	movlw	0x07
	xorwf	FDOW,w
	btfsc	STATUS,Z
	clrf	FDOW
NoFIncD
	btfss	BTN1,6
	goto	NoFDecD
	decf	FDOW,f
	movlw	0x06
	btfsc	FDOW,3
	movwf	FDOW
NoFDecD:

	movlw	DEBOUNCE_TIME
	movwf	KEY_DEB
SkipFBtn1RD:
	movf	BTN2,w
	andlw	b'00001010' ; buttons only on 2,1 and 2,3
	btfsc	STATUS,Z
	goto	ShowFastClock
	; read keys from BTN2
	btfss	BTN2,1
	goto	NoFResetS
	clrf	FSPD
NoFResetS:
	btfss	BTN2,3
	goto	NoFResetM
	clrf	FMIN
NoFResetM:	
	movlw	DEBOUNCE_TIME
	movwf	KEY_DEB
	goto	ShowFastClock
SkipFBtnRD:
	decf	KEY_DEB,f
ShowFastClock:



	incf	DPCOUNT,f

	btfsc	DPCOUNT,0
	bsf		STAT3,0		; half sec for point
	btfss	DPCOUNT,0
	bcf		STAT3,0		; half sec for point

	movf	FSPD,w
	btfsc	STATUS,Z
	goto	SkipFastCalc
	decfsz	FSTCOUNT,f
	goto	SkipFastCalc
	call	SpeedSteps
	movwf	FSTCOUNT

	incf	FMIN,f
	movlw	d'60'
	xorwf	FMIN,w
	btfss	STATUS,Z
	goto	SkipFastCalc
	clrf	FMIN

	incf	FHR,f
	movlw	d'24'
	xorwf	FHR,w
	btfss	STATUS,Z
	goto	SkipFastCalc
	clrf	FHR
	
	incf	FDOW,f
	movlw	d'7'
	xorwf	FDOW,w
	btfss	STATUS,Z
	goto	SkipFastCalc
	clrf	FDOW

SkipFastCalc:
	movf	FSPD,w	; speed
	movwf	TOBCD
	call	BIN2BCD
	swapf	FROMBCD,w
	andlw	0x03		; only 2 bites valid for Min 10
	call	HexTo7Seg
	movwf	STAT6
	movf	FROMBCD,w
	call	HexTo7Seg
	movwf	STAT7

	movf	FMIN,w	; mins
	movwf	TOBCD
	call	BIN2BCD
	swapf	FROMBCD,w
	andlw	0x07		; only 3 bites valid for Min 10
	call	HexTo7Seg
	movwf	STAT4
	movf	FROMBCD,w
	call	HexTo7Seg
	movwf	STAT5

	movf	FHR,w		; hours
	movwf	TOBCD
	call	BIN2BCD
	swapf	FROMBCD,w
	andlw	0x03		; only 2 bites valid for Hr 10
	call	HexTo7Seg
	movwf	STAT2
	movf	FROMBCD,w
	call	HexTo7Seg
	movwf	STAT3
	btfsc	DPCOUNT,0
	bsf		STAT3,0		; half sec for point

	movf	FDOW,w
	andlw	0x07		; only 3 bites valid for day of week
	call	DOWH
	movwf	STAT0
	movf	FDOW,w
	andlw	0x07		; only 3 bites valid for day of week
	call	DOWL
	movwf	STAT1
	return

NoClockMsg:
	movlw	SEG_E
	movwf	STAT3
	movlw	SEG_R
	movwf	STAT4
	movwf	STAT5
	movlw	SEG_1
	movwf	STAT6

	incf	DPCOUNT,f
	btfsc	DPCOUNT,1
	bsf		STAT3,7

	return

       
I2CWrite:
	bsf	SHDA,SCL
	bsf	PORTA,SCL		; for sure set SDA and SCL to high
	bsf	SHDA,SDA
	bsf	PORTA,SDA
	SEL_BANK_1
	bcf   	TRISA,SDA		; set SDA to output
	SEL_BANK_0

	bcf	SHDA,SDA
	bcf	PORTA,SDA		; START condition
	;DNOP
	DNOP
         
	bcf	SHDA,SCL
	bcf	PORTA,SCL		; reset clock to be ready for sending data

	movlw	0xA2			; address of device + write
	movwf	I2CTEMP			; into temp
	movlw	0x08
	movwf	I2CCOUNT
	call	I2CWrLoop		; send address

	movf	I2CADDR,w		; address of register
	movwf	I2CTEMP			; into temp
	movlw	0x08
	movwf	I2CCOUNT
	call	I2CWrLoop		; send address

	movf	I2CDATA,w		; data into register
	movwf	I2CTEMP			; into temp
	movlw	0x08
	movwf	I2CCOUNT
	call	I2CWrLoop		; send data
         
	bcf	SHDA,SDA
	bcf	I2CPORT,SDA			;
	bsf	SHDI2C,SCL			;
	bsf	I2CPORT,SCL			;
	bsf	SHDI2C,SDA			; STOP condition
	bsf	I2CPORT,SDA			; STOP condition
	return

         
I2CWrLoop:
	btfss	I2CTEMP,7
	bcf	SHDA,SDA
	btfss	I2CTEMP,7
	bcf	I2CPORT,SDA
	btfsc	I2CTEMP,7
	bsf	SHDA,SDA			; set bit to port
	btfsc	I2CTEMP,7
	bsf	I2CPORT,SDA			; set bit to port
	bsf	SHDA,SCL			;
	bsf	I2CPORT,SCL			; set clock
	DNOP
	bcf	SHDA,SCL			;
	bcf	I2CPORT,SCL			; end clock
	rlf	I2CTEMP,f		; prepare next bit
	decfsz	I2CCOUNT,f
	goto	I2CWrLoop
	bsf	SHDA,SDA			;
	bsf	I2CPORT,SDA		; set SDA to high to be able read
	SEL_BANK_1
	bsf   	TRISA,SDA		; set SDA to input
	SEL_BANK_0
	bsf	SHDA,SCL			;
	bsf	I2CPORT,SCL			; set clock
	btfsc	I2CPORT,SDA		; in a case of SDA=1 set status to Error
	bsf	I2CErr
	bcf	SHDA,SCL			;
	bcf	I2CPORT,SCL			; end clock
	SEL_BANK_1
	bcf   	TRISA,SDA		; set SDA to output
	SEL_BANK_0

	return
         
I2CRead:
	bsf	SHDI2C,SCL
	bsf	I2CPORT,SCL		; for sure set SDA and SCL to high
	bsf	SHDI2C,SDA
	bsf	I2CPORT,SDA
	SEL_BANK_1
	bcf   	TRISA,SDA		; set SDA to output
	SEL_BANK_0

	bcf	SHDI2C,SDA
	bcf	I2CPORT,SDA		; START condition
	DNOP
         
	bcf	SHDI2C,SCL
	bcf	I2CPORT,SCL		; reset clock to be ready for sending data

	movlw	0xA2			; address of device + write
	movwf	I2CTEMP			; into temp
	movlw	0x08
	movwf	I2CCOUNT
	call	I2CWrLoop		; send address

	movf	I2CADDR,w		; address of register
	movwf	I2CTEMP			; into temp
	movlw	0x08
	movwf	I2CCOUNT
	call	I2CWrLoop		; send address

	bcf	SHDI2C,SDA
	bcf	I2CPORT,SDA			;
	bsf	SHDI2C,SCL
	bsf	I2CPORT,SCL			;
	bsf	SHDI2C,SDA
	bsf	I2CPORT,SDA			; STOP condition
	DNOP
	bcf	SHDI2C,SDA
	bcf	I2CPORT,SDA		; START condition
	DNOP
	bcf	SHDI2C,SCL
	bcf	I2CPORT,SCL		; reset clock to be ready for sending data

	movlw	0xA3			; address of device + write
	movwf	I2CTEMP			; into temp
	movlw	0x08
	movwf	I2CCOUNT
	call	I2CWrLoop		; send address

	SEL_BANK_1
	bsf   	TRISA,SDA		; set SDA to input
	SEL_BANK_0

	movlw	0x08
	movwf	I2CCOUNT
	call	I2CRdLoop		; send data
         
	bcf	SHDI2C,SDA
	bcf	I2CPORT,SDA			;
	bsf	SHDI2C,SCL
	bsf	I2CPORT,SCL			;
	bsf	SHDI2C,SDA
	bsf	I2CPORT,SDA			; STOP condition
	movf	I2CTEMP,w
	movwf	I2CDATA
	return

I2CRdLoop:
	bsf	SHDI2C,SCL
	bsf	I2CPORT,SCL		; set clock
	bcf	STATUS,C
	btfsc	I2CPORT,SDA
	bsf	STATUS,C		; set bit from port
	bsf	SHDI2C,SDA
	bcf	I2CPORT,SCL		; end clock
	rlf	I2CTEMP,f		; prepare next bit
	decfsz	I2CCOUNT,f
	goto	I2CRdLoop
	bsf	SHDI2C,SDA
	bsf	I2CPORT,SDA		; set SDA to high to write no-ACK
	SEL_BANK_1
	bcf   	TRISA,SDA		; set SDA to output
	SEL_BANK_0
	bsf	SHDI2C,SCL
	bsf	I2CPORT,SCL			; set clock
	DNOP
	bcf	SHDI2C,SCL
	bcf	I2CPORT,SCL			; end clock
	return

	org	0x2110
;
	dt	" Fast   Clock "
	dt	(__VERNUM   & 0x0F)+0x30
	dt	__VERREV
;
	dt	"J.Fucik "
	dt	(__VERDAY   >> 4)  +0x30
	dt	(__VERDAY   & 0x0F)+0x30,"/"
	dt	(__VERMONTH >> 4)  +0x30
	dt	(__VERMONTH & 0x0F)+0x30,"/"
	dt	(__VERYEAR  >> 4)  +0x30
	dt	(__VERYEAR  & 0x0F)+0x30
;


	END
